@@ -1,5 +1,57 @@
 #include "stdafx.h"
 
+void extension_build_params(PyObject *params_py, params *params) {
+    size_t index;
+    char *key_s, *value_s, value_b;
+    long value_i;
+    float value_f;
+    PyObject *iterator, *element, *key, *value;
+
+    params->length = 0;
+
+    index = 0;
+    iterator = PyObject_GetIter(params_py);
+
+    while(TRUE) {
+        element = PyIter_Next(iterator);
+        if(element == NULL) { break; }
+
+        key = PyDict_GetItemString(element, "key");
+        value = PyDict_GetItemString(element, "value");
+
+#if PY_MAJOR_VERSION >= 3
+        key = PyUnicode_EncodeFSDefault(key);
+        key_s = PyBytes_AsString(key);
+#else
+        key_s = PyString_AsString(key);
+#endif
+
+        params->length++;
+        params->params[index].key = key_s;
+
+        if(PyBool_Check(value)) {
+            value_b = PyBool_Check(value);
+            params->params[index].value.boolean = value_b;
+        } else if(PyInt_Check(value)) {
+            value_i = PyInt_AsLong(value);
+            params->params[index].value.integer = value_i;
+        } else if(PyFloat_Check(value)) {
+            value_f = (float) PyFloat_AsDouble(value);
+            params->params[index].value.decimal = value_f;
+        } else if(PyString_Check(value)) {
+#if PY_MAJOR_VERSION >= 3
+            value = PyUnicode_EncodeFSDefault(value);
+            value_s = PyBytes_AsString(value);
+#else
+            value_s = PyString_AsString(value);
+#endif
+            params->params[index].value.string = value_s;
+        }
+
+        index++;
+    }
+};
+
 PyObject *extension_register(PyObject *self, PyObject *args) {
     Py_RETURN_NONE;
 };
@@ -10,16 +62,19 @@ PyObject *extension_unregister(PyObject *self, PyObject *args) {
 
 PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwargs) {
     int run_inline;
-    char demultiply, source_over;
+    char demultiply, source_over, destination_over;
     char *bottom_path, *top_path, *target_path, *algorithm;
-    PyObject *is_inline;
+    PyObject *is_inline, *params_py;
     struct pcv_image bottom, top;
+    param values[32];
+    params params = { 0, values };
     char *kwlist[] = {
         "bottom_path",
         "top_path",
         "target_path",
         "algorithm",
         "is_inline",
+        "params",
         NULL
     };
 
@@ -28,6 +83,7 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
     target_path = NULL;
     algorithm = NULL;
     is_inline = NULL;
+    params_py = NULL;
 
     if(PyArg_ParseTupleAndKeywords(
         args,
@@ -44,16 +100,23 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
     run_inline = is_inline == NULL ? 0 : PyBool_Check(is_inline);
     demultiply = is_multiplied(algorithm);
     source_over = strcmp(algorithm, "source_over") == 0;
+    destination_over = strcmp(algorithm, "destination_over") == 0;
+
+    if(params_py != NULL) {
+        extension_build_params(params_py, &params);
+    }
 
     Py_BEGIN_ALLOW_THREADS;
     VALIDATE_A(read_png(bottom_path, demultiply, &bottom), Py_BLOCK_THREADS Py_RETURN_NONE);
     VALIDATE_A(read_png(top_path, demultiply, &top), Py_BLOCK_THREADS Py_RETURN_NONE);
     if(source_over == TRUE) {
-        VALIDATE_A(blend_images_fast(&bottom, &top, algorithm), Py_BLOCK_THREADS Py_RETURN_NONE);
-    } else if(run_inline == TRUE) {
-        VALIDATE_A(blend_images_i(&bottom, &top, algorithm), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_A(blend_images_source_over_fast(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+    } else if(destination_over == TRUE) {
+        VALIDATE_A(blend_images_destination_over_fast(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+    } else if(run_inline == TRUE && source_over == TRUE) {
+        VALIDATE_A(blend_images_i(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
     } else {
-        VALIDATE_A(blend_images(&bottom, &top, algorithm), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_A(blend_images(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
     }
     VALIDATE_A(write_png(&bottom, demultiply, target_path), Py_BLOCK_THREADS Py_RETURN_NONE);
     VALIDATE_A(release_image(&top), Py_BLOCK_THREADS Py_RETURN_NONE);
@@ -65,18 +128,21 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
 
 PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwargs) {
     int run_inline;
-    char demultiply, source_over, use_algorithms;
+    char demultiply, source_over, destination_over, use_algorithms;
     char *bottom_path, *top_path, *target_path, *algorithm = NULL;
     struct pcv_image bottom, top;
     PyObject *paths, *iterator, *iteratorAlgorithms, *element, *first, *second,
-        *is_inline, *algorithms, *algorithm_o;
+        *is_inline, *algorithms, *algorithm_o, *params_py;
     Py_ssize_t size, algorithms_size;
+    param values[32];
+    params params = { 0, values };
     char *kwlist[] = {
         "paths",
         "target_path",
         "algorithm",
         "algorithms",
         "is_inline",
+        "params",
         NULL
     };
 
@@ -89,17 +155,19 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     algorithm = NULL;
     algorithms = NULL;
     is_inline = NULL;
+    params_py = NULL;
 
     if(PyArg_ParseTupleAndKeywords(
         args,
         kwargs,
-        "Os|sOO",
+        "Os|sOOO",
         kwlist,
         &paths,
         &target_path,
         &algorithm,
         &algorithms,
-        &is_inline
+        &is_inline,
+        &params_py
     ) == 0) { return NULL; }
 
     /* tries to determine the target values for the optional parameters
@@ -112,8 +180,14 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     be used to determine the kind of blend algorithm to use */
     demultiply = is_multiplied(algorithm);
     source_over = strcmp(algorithm, "source_over") == 0;
+    destination_over = strcmp(algorithm, "destination_over") == 0;
     use_algorithms = algorithms == NULL ? FALSE : TRUE;
-    run_inline = use_algorithms == TRUE ? FALSE : run_inline;
+
+    /* in case the parameters value has been provided, then it must be parsed
+    as a list of dictionaries containing the parameters */
+    if(params_py != NULL) {
+        extension_build_params(params_py, &params);
+    }
 
     /* retrieves the size of the paths sequence that has been provided
     this is relevant to determine the kind of operation to be performed */
@@ -167,6 +241,16 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     from the algorithms list*/
     if(use_algorithms) {
         algorithm_o = PyList_GetItem(algorithms, 0);
+
+        /* verifies if the provided algorithm value is a tuple
+        and if that's the case (parameters are provided for its
+        execution) then loads them properly */
+        if(PyTuple_Check(algorithm_o)) {
+            params_py = PyTuple_GetItem(algorithm_o, 1);
+            algorithm_o = PyTuple_GetItem(algorithm_o, 0);
+            extension_build_params(params_py, &params);
+        }
+
 #if PY_MAJOR_VERSION >= 3
         algorithm_o = PyUnicode_EncodeFSDefault(algorithm_o);
         algorithm = PyBytes_AsString(algorithm_o);
@@ -174,6 +258,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
         algorithm = PyString_AsString(algorithm_o);
 #endif
         source_over = strcmp(algorithm, "source_over") == 0;
+        destination_over = strcmp(algorithm, "destination_over") == 0;
     }
 
     /* retrieves the first two elements from the list to serve
@@ -207,17 +292,22 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
 
     if(source_over == TRUE) {
         VALIDATE_A(
-            blend_images_fast(&bottom, &top, algorithm),
+            blend_images_source_over_fast(&bottom, &top, algorithm, &params),
             Py_BLOCK_THREADS Py_RETURN_NONE
         );
-    } else if(run_inline == TRUE) {
+    } else if(destination_over == TRUE) {
         VALIDATE_A(
-            blend_images_i(&bottom, &top, algorithm),
+            blend_images_destination_over_fast(&bottom, &top, algorithm, &params),
+            Py_BLOCK_THREADS Py_RETURN_NONE
+        );
+    } else if(run_inline == TRUE && source_over == TRUE) {
+        VALIDATE_A(
+            blend_images_i(&bottom, &top, algorithm, &params),
             Py_BLOCK_THREADS Py_RETURN_NONE
         );
     } else {
         VALIDATE_A(
-            blend_images(&bottom, &top, algorithm),
+            blend_images(&bottom, &top, algorithm, &params),
             Py_BLOCK_THREADS Py_RETURN_NONE
         );
     }
@@ -255,6 +345,16 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
 
         if(use_algorithms) {
             algorithm_o = PyIter_Next(iteratorAlgorithms);
+
+            /* verifies if the provided algorithm value is a tuple
+            and if that's the case (parameters are provided for its
+            execution) then loads them properly */
+            if(PyTuple_Check(algorithm_o)) {
+                params_py = PyTuple_GetItem(algorithm_o, 1);
+                algorithm_o = PyTuple_GetItem(algorithm_o, 0);
+                extension_build_params(params_py, &params);
+            }
+
 #if PY_MAJOR_VERSION >= 3
             algorithm_o = PyUnicode_EncodeFSDefault(algorithm_o);
             algorithm = PyBytes_AsString(algorithm_o);
@@ -262,6 +362,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
             algorithm = PyString_AsString(algorithm_o);
 #endif
             source_over = strcmp(algorithm, "source_over") == 0;
+            destination_over = strcmp(algorithm, "destination_over") == 0;
         }
 
         Py_BEGIN_ALLOW_THREADS;
@@ -272,17 +373,22 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
         );
         if(source_over == TRUE) {
             VALIDATE_A(
-                blend_images_fast(&bottom, &top, algorithm),
+                blend_images_source_over_fast(&bottom, &top, algorithm, &params),
                 Py_BLOCK_THREADS Py_RETURN_NONE
             );
-        } else if(run_inline == TRUE) {
+        } else if(destination_over == TRUE) {
             VALIDATE_A(
-                blend_images_i(&bottom, &top, algorithm),
+                blend_images_destination_over_fast(&bottom, &top, algorithm, &params),
+                Py_BLOCK_THREADS Py_RETURN_NONE
+            );
+        } else if(run_inline == TRUE && source_over == TRUE) {
+            VALIDATE_A(
+                blend_images_i(&bottom, &top, algorithm, &params),
                 Py_BLOCK_THREADS Py_RETURN_NONE
             );
         } else {
             VALIDATE_A(
-                blend_images(&bottom, &top, algorithm),
+                blend_images(&bottom, &top, algorithm, &params),
                 Py_BLOCK_THREADS Py_RETURN_NONE
             );
         }
